package Mentat::Reporter::Engine;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Reporter::Engine

=head1 SYNOPSIS

 use Mentat::Reporter::Engine;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# All meaningfull reporting periods
my %PERIODS = (
    'none'   => 0,
    '5mins'  => 300,
    '10mins' => 600,
    '15mins' => 900,
    '20mins' => 1200,
    '30mins' => 1800,
    '1hr'    => 3600,
    '2hrs'   => (3600 * 2),
    '3hrs'   => (3600 * 3),
    '4hrs'   => (3600 * 4),
    '6hrs'   => (3600 * 6),
    '8hrs'   => (3600 * 8),
    '12hrs'  => (3600 * 12),
    '1day'   => 86400,
    '2days'  => (86400 * 2),
    '3days'  => (86400 * 3),
    '4days'  => (86400 * 4),
    '5days'  => (86400 * 5),
    '6days'  => (86400 * 6),
    '7days'  => (86400 * 7),
    '8days'  => (86400 * 8),
    '9days'  => (86400 * 9),
    '10days' => (86400 * 10),
    '1week'  => 604800,
    '2weeks' => (604800 * 2),
    '3weeks' => (604800 * 3),
    '4weeks' => (604800 * 4),
);

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Reporter::Engine->new();
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Reporter::Engine reference on success, croaks on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Mentat::Reporter::Engine instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Mentat::Reporter::Engine instance...
}

=item time_period($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub time_period($)
{
    my $invocant = shift;
    my $period = shift;
    confess "Invalid period '$period'" unless exists $PERIODS{$period};
    return $PERIODS{$period};
}

=item calculate_time_boundaries(%) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub calculate_time_boundaries(%)
{
    my $invocant = shift;
    my %args = @_;
    confess "Report period must be given as first argument" unless $args{period};
    confess "Invalid period '$args{period}'" unless exists $PERIODS{$args{period}};
    my ($th, $tl, $duration);

    $args{time}   = time() unless $args{time};
    $args{offset} = 0      unless $args{offset};
    $args{anchor} = 0      unless $args{anchor};
    $args{time}   = int($args{time});
    $args{offset} = int($args{offset});
    $args{anchor} = int($args{anchor});

    $duration = $PERIODS{$args{period}};
    confess "Invalid anchor time '$args{anchor}', anchor time must not be greater than report period" if $args{anchor} and $args{anchor} > $duration;

    $th = $args{time};
    #print STDERR "CTB1: $th (d: $duration)\n";
    if ($duration) {
        unless ($args{noadjust}) {
            $th -= ($th % $duration);
            $th += $duration;
            #print STDERR "CTB2: $th (d: $duration)\n";
        }
        if ($args{offset}) {
            $th -= ($duration * $args{offset});
            #print STDERR "CTB3: $th (d: $duration)\n";
        }
    }
    $tl = $th - $duration;

    return ($tl, $th);
}

=item check_timing($$;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub check_timing($$;$)
{
    my $invocant = shift;
    my ($period, $ctime, $last_run_ts) = @_;

    my $duration = $invocant->time_period($period);

    return 1 if not $last_run_ts or (($last_run_ts + $duration) < $ctime);
    return 0;
}

=item escape_keys(@) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on instance

=cut

sub escape_keys(@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @keys = @_;

    my @rv;
    foreach my $k (@keys) {
        $k =~ s/\s//g;
        $k =~ s/\./(dot)/g;
        $k =~ s/@/(at)/g;
        push(@rv, $k);
    }
    return @rv;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Reporter::Engine instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Reporter::Engine $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

# _msg_gen_keys($) [PROTECTED]
#
#  Usage     : my @keys = $self->_msg_gen_keys($);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on instance or given invalid arguments

sub _msg_gen_keys($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($msg,) = @_;

    my @keys;
    my $tids = $msg->path_values('_CESNET/EventTemplate');
    # If the '_CESNET/EventTemplate' attribute is not present, use the 'Category' instead
    unless (scalar @$tids) {
        $tids = $msg->path_values('Category');
    }
    # Fetch the list of source IP addressess
    my $srcs = $msg->path_values('Source/IP4');
    # Generate the keys
    foreach my $src (@$srcs) {
        push(@keys, join('+++', @$tids, $src));
    }
    return @keys;
}
=pod

=back

=cut

1;
